// Lrp Network Monitor: Simple network monitor written in Java
// Copyright (C) 2000 Martin Hejl
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import java.applet.Applet;
import java.util.Vector;
import java.awt.*;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * the applet that displays the network data see Readme.txt for a description
 * of supported parameters for the applet
 *
 * @author       Martin Hejl
 * @version      20001105 0.1beta
 */
public class LrpStatApplet extends Applet{
	protected NetworkConnection con;
	protected Vector v;
	protected int serverPort;
	protected Vector interfaceOptionsList;
	protected String strHostname;
	protected LrpStatModel myModel;
	protected InterfaceInfo myData;

	protected long deviceCount=0;

	protected Thread modelListener;


	protected void parseParameters(){
		String strDummy;
		InterfaceOptions opts;

		strHostname = getCodeBase().getHost();
		strDummy = getParameter("INFOPORT");

		if (strDummy == null) {
			return;
		}

		serverPort = (int)atol(strDummy);
		interfaceOptionsList = new Vector();
		int i;
		i = 0;

		// Read the parameters for the Interface and store those infos in the Vector "interfaceOptionsList"
		while (getParameter("DEV" + i + "_NAME") != null) {
			opts = new InterfaceOptions();
			opts.deviceName = getParameter("DEV" + i + "_NAME");
			opts.controlType = getParameter("DEV" + i + "_MODE");
			if (opts.controlType==null) opts.controlType = "numeric"; else opts.controlType = opts.controlType.toLowerCase();

			strDummy = getParameter("DEV" + i + "_TYPE");

			if (strDummy.toLowerCase().equals("static")) opts.deviceType = InterfaceOptions.DEVICE_TYPE_STATIC; else opts.deviceType = InterfaceOptions.DEVICE_TYPE_DYNAMIC;
			opts.maxThroughput = atol(getParameter("DEV" + i + "_MAX"));

			CommandInfo ci;
			int l=0;

			// Read the actions (if defined) for this interface
			while (getParameter("DEV" + i + "_ACTION" + l) != null){
				strDummy = getParameter("DEV" + i + "_ACTION" + l);

				ci = new CommandInfo();
				ci.caption = strDummy.substring(0, strDummy.indexOf(";"));
				strDummy = strDummy.substring(strDummy.indexOf(";")+1).trim();

				if (strDummy.toLowerCase().startsWith("get"))  {
					ci.commandType = CommandInfo.COMMAND_TYPE_GET;
					strDummy = strDummy.substring(3).trim();
					try {
						ci.commandURL = new URL("http://" + getCodeBase().getHost() + strDummy);
					} catch (java.net.MalformedURLException e) {
						System.err.println("Malformed URL " + strDummy + " for Parameter " + "DEV" + i + "_ACTION" + l);
						ci = null;
					}

				} else {
					if (strDummy.toLowerCase().startsWith("open")) {
						ci.commandType = CommandInfo.COMMAND_TYPE_OPEN;
						strDummy = strDummy.substring(4).trim();
						ci.commandHost = getCodeBase().getHost();
						ci.commandPort = atol(strDummy);
						if (ci.commandPort <= 0) {
							System.err.println("Illegal port " + strDummy + " for Parameter " + "DEV" + i + "_ACTION" + l);
							ci=null;
						}
					}
				}
				// If the could be generated, add it to the Vector
				if (ci != null) opts.actions.addElement((Object)ci);
				l++;
			}

			interfaceOptionsList.addElement((Object)opts);
			deviceCount++;
			i++;
		}

	}

 	public void init() {
		parseParameters();
		setLayout(new GridLayout(interfaceOptionsList.size(),0));
		myModel = new LrpStatModel(strHostname, serverPort, interfaceOptionsList);

		String controlType;
		String ifName;

		for (int i=0; i<interfaceOptionsList.size(); i++) {
			ifName = ((InterfaceOptions)interfaceOptionsList.elementAt(i)).deviceName;
			controlType = ((InterfaceOptions)interfaceOptionsList.elementAt(i)).controlType;
			if (controlType.equals("histogram")) {
				LrpStatHistGraph myGraph = new LrpStatHistGraph((InterfaceOptions)interfaceOptionsList.elementAt(i));
				myModel.AddModelListener(myGraph, ifName);
				add(myGraph);
			} else
			if (controlType.equals("bar")) {
				LrpStatBarGraph myGraph = new LrpStatBarGraph((InterfaceOptions)interfaceOptionsList.elementAt(i));
				myModel.AddModelListener(myGraph, ifName);
				add(myGraph);
			} else
			if (controlType.equals("line")) {
				LrpStatLineGraph myGraph = new LrpStatLineGraph((InterfaceOptions)interfaceOptionsList.elementAt(i));
				myModel.AddModelListener(myGraph, ifName);
				add(myGraph);
			} else
			{
			// numeric is the default     if (controlType.equals("numeric")) {
				LrpStatNumericGraph myGraph = new LrpStatNumericGraph((InterfaceOptions)interfaceOptionsList.elementAt(i));
				myModel.AddModelListener(myGraph, ifName);
				add(myGraph);
			}

		}

	}

	public void start(){
		modelListener = new Thread(myModel);
		modelListener.start();

	}


	public void stop() {
		myModel.stop();
		try {
			// Make sure the listener thread ends - otherwise,
			// because of the strange way Netscape handles resizing of windows
			// (sending lots of Start and Stop messages), the new thread might be
			// started before the old one has terminates - causing all kinds of
			// troubles
			modelListener.join();
		} catch (InterruptedException e) {}
		modelListener=null;

	}


	// Ok, not the perfect way to do this, but it works here (-1 is not a
	// probable value to occur, so we can misuse it for errors)
	private long atol(String s) {
		if (s==null) return(0);
		try {
			return(Long.valueOf(s).longValue());
		} catch (NumberFormatException ex) {
			return (-1);
		}
	}

}