import java.util.Vector;
import java.awt.*;
import java.awt.event.*;


// Lrp Network Monitor: Simple network monitor written in Java
// Copyright (C) 2000 Martin Hejl
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

/**
 * Implements a text only view of the device-data
 *
 * @author       Martin Hejl
 * @version      20001105 0.1beta
 */
public class LrpStatNumericGraph extends LrpStatAbstractGraph {


	LrpStatNumericGraph(InterfaceOptions opts) {
		super(opts);
	}

	public void paint(Graphics graph) {
		Image offScreenImg = createImage(getSize().width, getSize().height);
		Graphics g = offScreenImg.getGraphics();

		g.clearRect(0,0,getSize().width, getSize().height);

		// Draw borders and all the other stuff that's common between all three views
		super.paint(g);
		Rectangle clientRect = getClientRect();

		g.draw3DRect(clientRect.x,clientRect.y, clientRect.width,clientRect.height, true);

		FontMetrics fm;
		fm = g.getFontMetrics();

		int lineHeight = fm.getLeading() + fm.getMaxAscent() + fm.getMaxDescent();

		if (lastValue == null || value==null) {
			g.drawString("No data", clientRect.x, clientRect.y + lineHeight);
			graph.drawImage(offScreenImg,0,0,this);
			return;
		}
		int columnWidth = fm.stringWidth("In: ");
		if (fm.stringWidth("Out: ") > columnWidth) columnWidth = fm.stringWidth("Out: ");
		InterfaceInfo paintValue = subtract(value, lastValue);



		g.drawString("In:", clientRect.x, clientRect.y + lineHeight);
		g.drawString(String.valueOf(paintValue.getBytes(InterfaceInfo.DIRECTION_RECEIVE)), clientRect.x +columnWidth, clientRect.y  + lineHeight);

		g.drawString("Out:",clientRect.x , clientRect.y + 2*lineHeight);
		g.drawString(String.valueOf(paintValue.getBytes(InterfaceInfo.DIRECTION_TRANSMIT)), clientRect.x +columnWidth, clientRect.y + 2*lineHeight);

		graph.drawImage(offScreenImg,0,0,this);
	}

	public void dataArrived(LrpStatModelEvent e) {
		if (lastValue == null ) {
			lastValue=e.getInterfaceInfo();
		} else {
			lastValue = value;
		}


		value = e.getInterfaceInfo();


		repaint();
	}


}